/*******************************
 * GConfDialog
 * Author: Luca Viggiani
 * Version: 0.1
 * Date: 2009-12-07
 * License: GPL
 * Description: creates a gconf settings back end dialog 
 * 				automatically from a settings key root.
 * Usage: new GConfDialog (string key, string dialog_title)
 * Params:
 * 	    string key: the gconf key to connect to (i.e /apps/panel/applets/applet_46/prefs)
 * 		string dislog_title: text to be shown on window title bar
 *		string[] subkeys: used to choose the subkeys to hadle (comprising display order).
 * 			  if null all subkeys will be retieved.
 * 
 * TODO: Only BOOL and INT data types are implemented. STRING is implemented only partially (read)
 *******************************/

#include "gtkextra-gconfdialog.h"
#include <gconf/gconf-client.h>
#include <gconf/gconf.h>
#include <float.h>
#include <math.h>
#include <glib/gi18n-lib.h>




struct _GConfDialogPrivate {
	GConfClient* _default_client;
};

#define GCONF_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GCONF_DIALOG, GConfDialogPrivate))
enum  {
	GCONF_DIALOG_DUMMY_PROPERTY
};
static void _gconf_dialog_onCheckButtonActivated_gtk_button_clicked (GtkCheckButton* _sender, gpointer self);
static void _gconf_dialog_onSpinButtonValueChanged_gtk_spin_button_value_changed (GtkSpinButton* _sender, gpointer self);
static void _gconf_dialog_onResetButtonPressed_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void gconf_dialog_addEntry (GConfDialog* self, GConfEntry* entry);
static void gconf_dialog_onCheckButtonActivated (GConfDialog* self, GtkCheckButton* widget);
static void gconf_dialog_onSpinButtonValueChanged (GConfDialog* self, GtkSpinButton* widget);
static GConfSchema* gconf_dialog_get_schema (GConfDialog* self, GConfEntry* entry);
static void gconf_dialog_onResetButtonPressed (GConfDialog* self, GtkButton* widget);
static GObject * gconf_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gconf_dialog_parent_class = NULL;
static void gconf_dialog_finalize (GObject* obj);



GConfDialog* gconf_dialog_construct (GType object_type, const char* dialog_title) {
	GConfDialog * self;
	g_return_val_if_fail (dialog_title != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	gtk_window_set_title ((GtkWindow*) self, dialog_title);
	gtk_window_set_icon_name ((GtkWindow*) self, "gtk-preferences");
	return self;
}


GConfDialog* gconf_dialog_new (const char* dialog_title) {
	return gconf_dialog_construct (TYPE_GCONF_DIALOG, dialog_title);
}


GConfDialog* gconf_dialog_construct_with_subkeys (GType object_type, const char* key, const char* dialog_title, char** subkeys, int subkeys_length1) {
	GConfDialog * self;
	g_return_val_if_fail (key != NULL, NULL);
	g_return_val_if_fail (dialog_title != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	gtk_window_set_title ((GtkWindow*) self, dialog_title);
	gtk_window_set_icon_name ((GtkWindow*) self, "gtk-preferences");
	gconf_dialog_add_subkeys (self, key, subkeys, subkeys_length1);
	return self;
}


GConfDialog* gconf_dialog_new_with_subkeys (const char* key, const char* dialog_title, char** subkeys, int subkeys_length1) {
	return gconf_dialog_construct_with_subkeys (TYPE_GCONF_DIALOG, key, dialog_title, subkeys, subkeys_length1);
}


void gconf_dialog_add_key (GConfDialog* self, const char* key) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (key != NULL);
	inner_error = NULL;
	{
		GConfEntry* entry;
		entry = gconf_client_get_entry (self->priv->_default_client, key, NULL, TRUE, &inner_error);
		if (inner_error != NULL) {
			goto __catch3_g_error;
			goto __finally3;
		}
		if (entry != NULL) {
			gconf_dialog_addEntry (self, entry);
		}
	}
	goto __finally3;
	__catch3_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:47: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally3:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


void gconf_dialog_add_subkeys (GConfDialog* self, const char* key, char** subkeys, int subkeys_length1) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (key != NULL);
	inner_error = NULL;
	{
		GSList* prefs;
		prefs = gconf_client_all_entries (self->priv->_default_client, key, &inner_error);
		if (inner_error != NULL) {
			goto __catch4_g_error;
			goto __finally4;
		}
		if (subkeys == NULL) {
			{
				GSList* entry_collection;
				GSList* entry_it;
				entry_collection = prefs;
				for (entry_it = entry_collection; entry_it != NULL; entry_it = entry_it->next) {
					GConfEntry* entry;
					entry = (GConfEntry*) entry_it->data;
					{
						gconf_dialog_addEntry (self, entry);
					}
				}
			}
		} else {
			{
				char** subkey_collection;
				int subkey_collection_length1;
				int subkey_it;
				subkey_collection = subkeys;
				subkey_collection_length1 = subkeys_length1;
				for (subkey_it = 0; subkey_it < subkeys_length1; subkey_it = subkey_it + 1) {
					const char* subkey;
					subkey = subkey_collection[subkey_it];
					{
						char* _tmp1;
						char* _tmp0;
						_tmp1 = NULL;
						_tmp0 = NULL;
						gconf_dialog_add_key (self, _tmp1 = g_strconcat (_tmp0 = g_strconcat (key, "/", NULL), subkey, NULL));
						_tmp1 = (g_free (_tmp1), NULL);
						_tmp0 = (g_free (_tmp0), NULL);
					}
				}
			}
		}
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:62: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally4:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void _gconf_dialog_onCheckButtonActivated_gtk_button_clicked (GtkCheckButton* _sender, gpointer self) {
	gconf_dialog_onCheckButtonActivated (self, _sender);
}


static void _gconf_dialog_onSpinButtonValueChanged_gtk_spin_button_value_changed (GtkSpinButton* _sender, gpointer self) {
	gconf_dialog_onSpinButtonValueChanged (self, _sender);
}


static void _gconf_dialog_onResetButtonPressed_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	gconf_dialog_onResetButtonPressed (self, _sender);
}


static void gconf_dialog_addEntry (GConfDialog* self, GConfEntry* entry) {
	GError * inner_error;
	GConfSchema* schema;
	const char* tooltip;
	GtkBox* row;
	GtkImage* info;
	GtkLabel* label;
	GtkWidget* widget;
	GtkEventBox* _tmp11;
	GtkButton* button;
	GtkImage* _tmp12;
	g_return_if_fail (self != NULL);
	g_return_if_fail (entry != NULL);
	inner_error = NULL;
	schema = gconf_dialog_get_schema (self, entry);
	tooltip = gconf_schema_get_long_desc (schema);
	row = (GtkBox*) g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
	info = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_BUTTON));
	gtk_widget_set_tooltip_text ((GtkWidget*) info, tooltip);
	gtk_box_pack_start (row, (GtkWidget*) info, FALSE, FALSE, (guint) 2);
	label = g_object_ref_sink ((GtkLabel*) gtk_label_new (gconf_schema_get_short_desc (schema)));
	gtk_label_set_justify (label, GTK_JUSTIFY_LEFT);
	gtk_widget_set_tooltip_text ((GtkWidget*) label, tooltip);
	gtk_box_pack_start (row, (GtkWidget*) label, FALSE, FALSE, (guint) 2);
	widget = NULL;
	switch (gconf_schema_get_type (schema)) {
		case GCONF_VALUE_BOOL:
		{
			GtkWidget* _tmp0;
			gboolean _tmp1;
			GtkWidget* _tmp2;
			GtkWidget* _tmp3;
			_tmp0 = NULL;
			widget = (_tmp0 = (GtkWidget*) g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new ()), (widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL)), _tmp0);
			_tmp1 = gconf_client_get_bool (self->priv->_default_client, entry->key, &inner_error);
			if (inner_error != NULL) {
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
				(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
				(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return;
			}
			_tmp2 = NULL;
			gtk_toggle_button_set_active ((GtkToggleButton*) (_tmp2 = widget, GTK_IS_CHECK_BUTTON (_tmp2) ? ((GtkCheckButton*) _tmp2) : NULL), _tmp1);
			_tmp3 = NULL;
			g_signal_connect_object ((GtkButton*) (_tmp3 = widget, GTK_IS_CHECK_BUTTON (_tmp3) ? ((GtkCheckButton*) _tmp3) : NULL), "clicked", (GCallback) _gconf_dialog_onCheckButtonActivated_gtk_button_clicked, self, 0);
			break;
		}
		case GCONF_VALUE_STRING:
		{
			GtkWidget* _tmp4;
			const char* _tmp5;
			GtkWidget* _tmp6;
			_tmp4 = NULL;
			widget = (_tmp4 = (GtkWidget*) g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL)), _tmp4);
			_tmp5 = gconf_client_get_string (self->priv->_default_client, entry->key, &inner_error);
			if (inner_error != NULL) {
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
				(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
				(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return;
			}
			_tmp6 = NULL;
			gtk_entry_set_text ((_tmp6 = widget, GTK_IS_ENTRY (_tmp6) ? ((GtkEntry*) _tmp6) : NULL), _tmp5);
			break;
		}
		case GCONF_VALUE_INT:
		{
			GtkWidget* _tmp7;
			gint _tmp8;
			GtkWidget* _tmp9;
			GtkWidget* _tmp10;
			_tmp7 = NULL;
			widget = (_tmp7 = (GtkWidget*) g_object_ref_sink ((GtkSpinButton*) gtk_spin_button_new_with_range ((double) (-100), (double) 200, (double) 1)), (widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL)), _tmp7);
			_tmp8 = gconf_client_get_int (self->priv->_default_client, entry->key, &inner_error);
			if (inner_error != NULL) {
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
				(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
				(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return;
			}
			_tmp9 = NULL;
			gtk_spin_button_set_value ((_tmp9 = widget, GTK_IS_SPIN_BUTTON (_tmp9) ? ((GtkSpinButton*) _tmp9) : NULL), (double) _tmp8);
			_tmp10 = NULL;
			g_signal_connect_object ((_tmp10 = widget, GTK_IS_SPIN_BUTTON (_tmp10) ? ((GtkSpinButton*) _tmp10) : NULL), "value-changed", (GCallback) _gconf_dialog_onSpinButtonValueChanged_gtk_spin_button_value_changed, self, 0);
			break;
		}
		default:
		{
			(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
			(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
			(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
			return;
		}
	}
	gtk_widget_set_tooltip_text (widget, tooltip);
	g_object_set ((GtkObject*) widget, "user-data", entry, NULL);
	_tmp11 = NULL;
	gtk_box_pack_start (row, (GtkWidget*) (_tmp11 = g_object_ref_sink ((GtkEventBox*) gtk_event_box_new ())), TRUE, TRUE, (guint) 2);
	(_tmp11 == NULL) ? NULL : (_tmp11 = (g_object_unref (_tmp11), NULL));
	gtk_box_pack_start (row, widget, FALSE, FALSE, (guint) 2);
	button = g_object_ref_sink ((GtkButton*) gtk_button_new ());
	gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Reset to the default value"));
	_tmp12 = NULL;
	gtk_button_set_image (button, (GtkWidget*) (_tmp12 = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock ("gtk-clear", GTK_ICON_SIZE_BUTTON))));
	(_tmp12 == NULL) ? NULL : (_tmp12 = (g_object_unref (_tmp12), NULL));
	g_object_set ((GtkObject*) button, "user-data", widget, NULL);
	g_signal_connect_object (button, "clicked", (GCallback) _gconf_dialog_onResetButtonPressed_gtk_button_clicked, self, 0);
	gtk_box_pack_start (row, (GtkWidget*) button, FALSE, FALSE, (guint) 2);
	gtk_widget_show_all ((GtkWidget*) row);
	gtk_box_pack_start ((GtkBox*) ((GtkDialog*) self)->vbox, (GtkWidget*) row, TRUE, TRUE, (guint) 2);
	(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
	(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
	(button == NULL) ? NULL : (button = (g_object_unref (button), NULL));
}


static void gconf_dialog_onCheckButtonActivated (GConfDialog* self, GtkCheckButton* widget) {
	GError * inner_error;
	void* _tmp0;
	GConfEntry* entry;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	inner_error = NULL;
	entry = (GConfEntry*) (g_object_get ((GtkObject*) widget, "user-data", &_tmp0, NULL), _tmp0);
	{
		gconf_client_set_bool (self->priv->_default_client, entry->key, gtk_toggle_button_get_active ((GtkToggleButton*) widget), &inner_error);
		if (inner_error != NULL) {
			goto __catch5_g_error;
			goto __finally5;
		}
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:124: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally5:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void gconf_dialog_onSpinButtonValueChanged (GConfDialog* self, GtkSpinButton* widget) {
	GError * inner_error;
	void* _tmp0;
	GConfEntry* entry;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	inner_error = NULL;
	entry = (GConfEntry*) (g_object_get ((GtkObject*) widget, "user-data", &_tmp0, NULL), _tmp0);
	{
		gconf_client_set_int (self->priv->_default_client, entry->key, (gint) gtk_spin_button_get_value (widget), &inner_error);
		if (inner_error != NULL) {
			goto __catch6_g_error;
			goto __finally6;
		}
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_warning ("gtkextra-gconfdialog.vala:133: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally6:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static GConfSchema* gconf_dialog_get_schema (GConfDialog* self, GConfEntry* entry) {
	GError * inner_error;
	const char* schema_name;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (entry != NULL, NULL);
	inner_error = NULL;
	schema_name = gconf_entry_get_schema_name (entry);
	if (schema_name == NULL) {
		GConfSchema* rt;
		g_critical ("gtkextra-gconfdialog.vala:141: schema not found for entry %s", gconf_entry_get_key (entry));
		rt = gconf_schema_new ();
		gconf_schema_set_short_desc (rt, gconf_entry_get_key (entry));
		gconf_schema_set_long_desc (rt, "This key lacks a schema");
		gconf_schema_set_type (rt, GCONF_VALUE_STRING);
		return rt;
	} else {
		GConfSchema* _tmp1;
		_tmp1 = gconf_client_get_schema (self->priv->_default_client, schema_name, &inner_error);
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return NULL;
		}
		return _tmp1;
	}
}


static void gconf_dialog_onResetButtonPressed (GConfDialog* self, GtkButton* widget) {
	GtkWidget* _tmp1;
	void* _tmp0;
	GtkWidget* target;
	void* _tmp2;
	GConfEntry* entry;
	GConfSchema* schema;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_tmp1 = NULL;
	target = (_tmp1 = GTK_WIDGET ((g_object_get ((GtkObject*) widget, "user-data", &_tmp0, NULL), _tmp0)), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	entry = (GConfEntry*) (g_object_get ((GtkObject*) target, "user-data", &_tmp2, NULL), _tmp2);
	schema = gconf_dialog_get_schema (self, entry);
	switch (gconf_schema_get_type (schema)) {
		case GCONF_VALUE_BOOL:
		{
			GtkWidget* _tmp3;
			_tmp3 = NULL;
			gtk_toggle_button_set_active ((GtkToggleButton*) (_tmp3 = target, GTK_IS_CHECK_BUTTON (_tmp3) ? ((GtkCheckButton*) _tmp3) : NULL), gconf_value_get_bool (gconf_schema_get_default_value (schema)));
			break;
		}
		case GCONF_VALUE_STRING:
		{
			GtkWidget* _tmp4;
			_tmp4 = NULL;
			gtk_entry_set_text ((_tmp4 = target, GTK_IS_ENTRY (_tmp4) ? ((GtkEntry*) _tmp4) : NULL), gconf_value_get_string (gconf_schema_get_default_value (schema)));
			break;
		}
		case GCONF_VALUE_INT:
		{
			GtkWidget* _tmp5;
			_tmp5 = NULL;
			gtk_spin_button_set_value ((_tmp5 = target, GTK_IS_SPIN_BUTTON (_tmp5) ? ((GtkSpinButton*) _tmp5) : NULL), (double) gconf_value_get_int (gconf_schema_get_default_value (schema)));
			break;
		}
	}
	(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
	(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
}


static GObject * gconf_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GConfDialogClass * klass;
	GObjectClass * parent_class;
	GConfDialog * self;
	klass = GCONF_DIALOG_CLASS (g_type_class_peek (TYPE_GCONF_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GCONF_DIALOG (obj);
	{
		GConfClient* _tmp1;
		GConfClient* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		self->priv->_default_client = (_tmp1 = (_tmp0 = gconf_client_get_default (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_default_client == NULL) ? NULL : (self->priv->_default_client = (g_object_unref (self->priv->_default_client), NULL)), _tmp1);
		/*vbox.width_request = 320;*/
		gtk_dialog_add_button ((GtkDialog*) self, "gtk-close", (gint) GTK_RESPONSE_CLOSE);
	}
	return obj;
}


static void gconf_dialog_class_init (GConfDialogClass * klass) {
	gconf_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GConfDialogPrivate));
	G_OBJECT_CLASS (klass)->constructor = gconf_dialog_constructor;
	G_OBJECT_CLASS (klass)->finalize = gconf_dialog_finalize;
}


static void gconf_dialog_instance_init (GConfDialog * self) {
	self->priv = GCONF_DIALOG_GET_PRIVATE (self);
}


static void gconf_dialog_finalize (GObject* obj) {
	GConfDialog * self;
	self = GCONF_DIALOG (obj);
	(self->priv->_default_client == NULL) ? NULL : (self->priv->_default_client = (g_object_unref (self->priv->_default_client), NULL));
	G_OBJECT_CLASS (gconf_dialog_parent_class)->finalize (obj);
}


GType gconf_dialog_get_type (void) {
	static GType gconf_dialog_type_id = 0;
	if (gconf_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GConfDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gconf_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GConfDialog), 0, (GInstanceInitFunc) gconf_dialog_instance_init, NULL };
		gconf_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "GConfDialog", &g_define_type_info, 0);
	}
	return gconf_dialog_type_id;
}




